package hk.hku.cecid.phoenix.hermes.webmin.app;

import java.io.IOException;
import java.io.InputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.MalformedURLException;
import java.net.HttpURLConnection;
import java.net.ProtocolException;

import java.util.Hashtable;

public abstract class AbstractWebminRequestHandler {
    
    private String serverRootRealPath;
    private XMLTransformer transformer;
    protected ConfigFileHandler configHandler;
    private String authUser, authPwd, monitoringMshUrl;
    
            
    protected AbstractWebminRequestHandler(String serverRootRealPath) throws WebminException {
        transformer = new XMLTransformer(serverRootRealPath);
        configHandler = new ConfigFileHandler(serverRootRealPath);        
    }
        
    public String issueRequest(String moduleName, String fnName)
         throws WebminException {
        return issueRequest(moduleName, fnName, null);
    }
    
    public String issueRequest(String moduleName, String fnName, 
        Hashtable params) throws WebminException {
        authUser = configHandler.getAuthName();
        authPwd = configHandler.getAuthPassword();
        monitoringMshUrl = configHandler.getMonitoringMshUrl();        
        
        HttpURLConnection webminConn = null;
        int responseCode;
        String responseMsg = null;
        
        try {
            URL webminURL = new URL(monitoringMshUrl+"/webmin");
            webminConn = (HttpURLConnection) webminURL.openConnection();
        } catch (MalformedURLException e) {
            throw new WebminException("Malformed monitoring Msh and webmin URL<br>"+
                e.getClass()+":"+e.getMessage());
        } catch (IOException e) {
            throw new WebminException("Error when opening webmin servlet connection<br>"+
                e.getClass()+":"+e.getMessage());
        }    
                
        webminConn.setDoOutput(true);
        webminConn.setDoInput(true);
                
        webminConn.setRequestProperty("Authenticate_user", authUser);
        webminConn.setRequestProperty("Authenticate_password", authPwd);
        webminConn.setRequestProperty("ModuleName", moduleName);
        webminConn.setRequestProperty("FunctionName", fnName);
        
        if (params != null) {
            Object keys[] = params.keySet().toArray();
            for (int i = 0; i < keys.length; i++) {
                webminConn.setRequestProperty(keys[i].toString(),
                    params.get(keys[i]).toString());
            }
        }
               
        try {
            webminConn.setRequestMethod("POST");
            webminConn.setRequestProperty("Content-length", "0");
        } catch (ProtocolException e) {
            throw new WebminException("Error when setting \"POST\" protocol for the webmin servlet connection<br>"+
                    e.getClass()+":"+e.getMessage());
        }   
        
        try {
            InputStream stream;
            if (webminConn.getResponseCode() == 200) {
                stream = webminConn.getInputStream();
            } else {
                stream = webminConn.getErrorStream();
            }
            StringBuffer result = new StringBuffer();
            BufferedReader in = new BufferedReader(
                            new InputStreamReader(stream));
            while (true) {
                String line = in.readLine();
                if (line == null) {
                    break;
                }
                result.append(line);
            }
            in.close();             
            webminConn.disconnect();
            
            if (webminConn.getHeaderField("ResponseResult") == null) {
                throw new WebminException("Error response from the remote web server<br>"+
                    result.toString());
            } else if (webminConn.getHeaderField("ResponseResult").equals("Error")) {    
                throw new WebminException("Error response from the webmin servlet<br>"+
                    result.toString());
            } else {                               
                return transformer.getTransformedHtml(result.toString(),
                    moduleName, fnName);
            }
        } catch (IOException e) {
            e.printStackTrace();
            return "Exception when reading result from the webmin servlet<br>"
                +e.getClass()+":"+e.getMessage();
        }        
    }
    
    public void setMshUrl(String mshURLString, String authName, String authPwd)
        throws WebminException {        
        configHandler.setMonitoringMshUrl(mshURLString, authName, authPwd);        
    }
    
    public String getMshUrl() throws WebminException{
        return configHandler.getMonitoringMshUrl();
    }
    
    public boolean isMshConnectionAlive() throws WebminException {        
        HttpURLConnection mshConn = null;
                
        try {
            URL mshURL = new URL(getMshUrl());
            mshConn = (HttpURLConnection) mshURL.openConnection();
        } catch (MalformedURLException e) {
            throw new WebminException("Malformed monitoring Msh URL<br>"+
                e.getClass()+":"+e.getMessage());
        } catch (IOException e) {
            throw new WebminException("Error when opening msh servlet connection<br>"+
                e.getClass()+":"+e.getMessage());
        } 
        
        try {
            if (mshConn.getResponseCode() == 200) {
                return true;
            } else {
               return false;
            }            
        } catch (IOException e) {
            throw new WebminException("Exception when reading response code from the msh servlet<br>"
                +e.getClass()+":"+e.getMessage());
        }
                
    }
    
     
}