package hk.hku.cecid.phoenix.hermes.webmin.app;

import java.io.IOException;
import java.io.File;
import java.io.FileWriter;

import org.jdom.*;
import org.jdom.input.*;
import org.jdom.output.*;

class ConfigFileHandler {
    String HANDLER_CONFIG_FILE = "WEB-INF/webminApp.properties.xml";
    private Document doc;
    private String authName, authPassword, monitorMsh;
    private Element authNameElement, authPasswdElement, monitorMshElement;
    private SAXBuilder builder;
    private XMLOutputter outputter;
    
    ConfigFileHandler(String serverRootRealPath) throws WebminException {
        HANDLER_CONFIG_FILE = serverRootRealPath+"/"+HANDLER_CONFIG_FILE;
        File tmp = new File(HANDLER_CONFIG_FILE);
        builder = new SAXBuilder();
        outputter = new XMLOutputter();
        reload();
    }
    
    private void reload() throws WebminException {
        try {
            doc = builder.build(new File(HANDLER_CONFIG_FILE));
        } catch (JDOMException e) {
            throw new WebminException("Error when parsing the webminApp.properties.xml File<br>"+
                e.getClass()+":"+e.getMessage());
        }
        Element rootElement = doc.getRootElement();
        monitorMshElement = rootElement.getChild("MonitoringMshUrl");
        monitorMsh = rootElement.getChildText("MonitoringMshUrl");
        
        Element authElement = rootElement.getChild("WebminServletAuthentication");
        authNameElement = authElement.getChild("Name");
        authName = authElement.getChildText("Name");
        authPasswdElement = authElement.getChild("Password");
        authPassword = authElement.getChildText("Password");
    }
    
    void setMonitoringMshUrl(String mshUrl, String name, String pwd) throws WebminException {
        this.monitorMsh = mshUrl;
        authName = name;
        authPassword = pwd;
        saveConfigFile();
    }
    
    String getMonitoringMshUrl() throws WebminException {        
        reload();
        return monitorMsh;
    }
    
    String getAuthName() throws WebminException{        
        reload();
        return authName;
    }
    
    String getAuthPassword() throws WebminException {        
        reload();
        return authPassword;
    }
    
    private void saveConfigFile() throws WebminException{
        monitorMshElement.setText(monitorMsh);
        authNameElement.setText(authName);
        authPasswdElement.setText(authPassword);
        try {
            FileWriter writer = new FileWriter(HANDLER_CONFIG_FILE);
            outputter.output(doc, writer);
            writer.close();
        } catch (IOException e) {
            throw new WebminException ("Error when writing the webApp.properties.xml<br>"+
                e.getClass()+":"+e.getMessage());
        }
    }
}