package hk.hku.cecid.phoenix.hermes.webmin.app;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.*;
import sun.misc.BASE64Encoder;
import java.io.File;
import java.io.FileOutputStream;
import java.util.HashSet;

import org.apache.xpath.XPathAPI;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.traversal.NodeIterator;

// Imported JAVA API for XML Parsing 1.0 classes
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException; 

// Imported Serializer classes
import javax.xml.transform.*;
import javax.xml.transform.stream.*;
import javax.xml.transform.dom.*;


public class UserManager {
    
    private HashSet userSet = new HashSet();
    private File passwdFile;    
    private Document doc;
    private Element rootElement;
    private TransformerFactory tFactory;
    private Transformer transformer;
    private DocumentBuilderFactory dfactory;
    private DocumentBuilder db;
    private MessageDigest md;
    private BASE64Encoder encoder;
     
    public static final String DEFAULT_ADMIN = "default_admin";
               
    public UserManager(String serverRootRealPath) throws WebminException{       
        try {
            passwdFile = new File(serverRootRealPath+"/acl/passwd.xml");
            md = MessageDigest.getInstance("MD5");
            dfactory = DocumentBuilderFactory.newInstance();
            dfactory.setNamespaceAware(true);            
            db = dfactory.newDocumentBuilder();            
            reload();
            tFactory = TransformerFactory.newInstance();
            transformer = tFactory.newTransformer();
        } catch (Exception e) {
            throw new WebminException("Initialization error for the user manager object<br>"+
                e.getClass()+":"+e.getMessage());
        }
    }

    private void reload() {
        try {
            userSet.clear();            
            doc = db.parse(passwdFile);
            NodeIterator nl = XPathAPI.selectNodeIterator(doc, "//id");
            Node n;
            while ((n = nl.nextNode())!= null) {          
                String user = n.getFirstChild().getNodeValue();                
                userSet.add(user);                
            }        
            rootElement = doc.getDocumentElement();        
        } catch (Exception e) {            
            e.printStackTrace();
        }
    }
            
    public String[] listUser() {        
        reload();
        return toStringArray(userSet.toArray());
    }
    
    public boolean isUserExists(String id) {
        reload();
        return (userSet.contains(id));
    }
    
    public boolean editUser(String id, String[] moduleName) {                
        Node userNode = findUser(id);
        if (userNode != null) {
            Node passNode = getSiblingNode(userNode);
            String cryptedPasswd = passNode.getFirstChild().getNodeValue();
            Node dateNode = getSiblingNode(passNode);
            String loginDate = null;
            if (dateNode.getFirstChild() != null) {
                loginDate = dateNode.getFirstChild().getNodeValue();
            }
            if (removeUser(id)) {
                return realAddUser(id, cryptedPasswd, moduleName, loginDate);
            } else {
                return false;
            }
        } else {
            return false;
        }            
    }
    
    public boolean editUser(String id, String passwd, String[] moduleName) {
        Node userNode = findUser(id);
        if (userNode != null) {
            String cryptedPasswd = encrypt(passwd);
            Node passNode = getSiblingNode(userNode);
            Node dateNode = getSiblingNode(passNode);
            String loginDate = null;
            if (dateNode.getFirstChild() != null) {
                loginDate = dateNode.getFirstChild().getNodeValue();
            }
            if (removeUser(id)) {
                return realAddUser(id, cryptedPasswd, moduleName, loginDate);
            } else {
                return false;
            }
        } else {
            return false;
        }             
    }
    
    public boolean addUser(String id, String passwd, String[] moduleName) {
        return realAddUser(id, encrypt(passwd), moduleName, null);
    }
        
    public boolean removeUser(String id) {        
        reload();
        if (id == null || id.trim().equals("") ||
            !userSet.contains(id)) {
            return false;
        }
        Node userNode = findUser(id);
        if (userNode != null) {
            Node parentNode = userNode.getParentNode();
            rootElement.removeChild(parentNode);
            toFile();
            userSet.remove(id);        
            return true;
        }        
        return false;
    }
    
    public String getLastLoginDate(String id) {
        reload();
        Node userNode = findUser(id);
        if (userNode != null) {
            Node passNode = getSiblingNode(userNode);
            Node dateNode = getSiblingNode(passNode);
            Node oldDateNode = null;
            if (dateNode.getFirstChild() != null) {
                oldDateNode = dateNode.getFirstChild();
            }
            String dateString = "N/A";
            if (oldDateNode != null) {
                dateString = oldDateNode.getNodeValue();
                dateNode.replaceChild(doc.createTextNode((new Date()).toString()), oldDateNode);
            } else {
                dateNode.appendChild(doc.createTextNode((new Date()).toString()));
            }
            toFile();            
            if (dateString != null && !dateString.trim().equals("")) {
                return dateString;
            } else {
                return "N/A";
            }                        
        } else {
            return null;
        }                    
    }
    
    public boolean canLogin(String id, String passwd) {    
        reload();
        Node userNode = findUser(id);
        if (userNode != null) {
            Node passNode = getSiblingNode(userNode);                    
            String pwd = passNode.getFirstChild().getNodeValue();
            if (pwd.equals(encrypt(passwd))) {                        
                return true;
            } else {
                return false;
            }            
        } else {
            return false;
        }                    
    }
    
    public String[] getAccessModule(String id) {
        reload();
        Node userNode = findUser(id);
        ArrayList moduleList = new ArrayList();
        if (userNode != null) {
            Node passNode = getSiblingNode(userNode);                    
            Node dateNode = getSiblingNode(passNode);                    
            Node checkNode = getSiblingNode(dateNode);
            while (checkNode != null) {
                String module = checkNode.getNodeName();
                moduleList.add(module);
                checkNode = getSiblingNode(checkNode);
            }
            return toStringArray(moduleList.toArray());
        }
        return null;                    
    }
    
    public boolean canAccessModule(String id, String moduleName) {        
        reload();
        Node userNode = findUser(id);
        if (userNode != null) {
            Node passNode = getSiblingNode(userNode);
            Node dateNode = getSiblingNode(passNode);
            Node checkNode = getSiblingNode(dateNode);
            while (checkNode != null) {
                String module = checkNode.getNodeName();
                if (module.equals(moduleName)) {
                    return true;
                } else {
                    checkNode = getSiblingNode(checkNode);
                }
            }
            return false;                
        }
        return false;                    
    }
    
    private boolean realAddUser(String id, String cryptedPasswd, String[] moduleName, String loginDate) {
        reload();        
        if (id == null || id.trim().equals("") ||
            userSet.contains(id) || cryptedPasswd == null ||
            cryptedPasswd.trim().equals("")) {
            return false;
        }        
        Element newUserElement = doc.createElement("User"); 	
        Element newIdElement = doc.createElement("id");
        newIdElement.appendChild(doc.createTextNode(id));
        Element newPasswdElement = doc.createElement("passwd");
        newPasswdElement.appendChild(doc.createTextNode(cryptedPasswd));
        Element newLoginDateElement = doc.createElement("loginDate");
        if (loginDate != null) {
            newLoginDateElement.appendChild(doc.createTextNode(loginDate));
        }
        newUserElement.appendChild(newIdElement);
        newUserElement.appendChild(newPasswdElement);
        newUserElement.appendChild(newLoginDateElement);
        
        if (moduleName != null) {
            for (int i = 0; i < moduleName.length; i++) {
                Element newModuleElement = doc.createElement(moduleName[i]);
                newUserElement.appendChild(newModuleElement);
            }
        }        
        rootElement.appendChild(newUserElement);        
        toFile();
        userSet.add(id);
        return true;
    }    
    
    
    private void toFile() {
        try {
            DOMSource source = new DOMSource(doc);
            StreamResult result = new StreamResult(new FileOutputStream(passwdFile));
            transformer.transform(source, result);
        } catch (Exception e) {
            System.err.println("Error when writing the passwd file");
            e.printStackTrace();
        }
    }
    
    private Node findUser(String id) {
        reload();
        try {
            NodeIterator nl = XPathAPI.selectNodeIterator(doc, "//id");
            Node n;
            while ((n = nl.nextNode())!= null) {           
                String user = n.getFirstChild().getNodeValue();
                if (user.equals(id)) {
                    return n;
                }
            }
        } catch (TransformerException e) {
            return null;
        }
        return null;
    }
        
    
    private Node getSiblingNode(Node node) {
        node = node.getNextSibling();
        while (isTextNode(node)) {
            node = node.getNextSibling();                        
        }
        return node;
    }
    
    private boolean isTextNode(Node n) {
        if (n == null)
            return false;
        short nodeType = n.getNodeType();
        return nodeType == Node.CDATA_SECTION_NODE || nodeType == Node.TEXT_NODE;
    }
    
    private String[] toStringArray(Object[] obj) {
        String strings[] = new String[obj.length];
        for (int i = 0; i < strings.length; i++) {
            strings[i] = obj[i].toString();
        }
        return strings;
    }
    
    private String encrypt(String str) {        
        md.update(str.getBytes());
        encoder = new BASE64Encoder();
        return encoder.encode(md.digest());        
    }    
}