package hk.hku.cecid.phoenix.hermes.webmin.app;

import java.io.File;
import java.io.IOException;
import java.io.ByteArrayInputStream;
import java.io.StringWriter;
import java.net.URL;
import java.net.MalformedURLException;
import java.util.*;
import org.apache.xpath.XPathAPI;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.traversal.NodeIterator;

// Imported JAVA API for XML Parsing 1.0 classes
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException; 
import org.xml.sax.SAXException;

// Imported Serializer classes
import javax.xml.transform.*;
import javax.xml.transform.stream.*;
import javax.xml.transform.dom.*;


// Imported Serializer classes
import javax.xml.transform.*;
import javax.xml.transform.stream.*;
import javax.xml.transform.dom.*;

class XMLTransformer {
    private String xslRootRealPath;
    private File xslMapping;
    private Document doc;
    private Hashtable moduleTable;
    private DocumentBuilderFactory dfactory;
    private DocumentBuilder db; 
    private TransformerFactory tf;
        
    XMLTransformer(String serverRootRealPath) throws WebminException {
        this.xslRootRealPath = serverRootRealPath+"/transform";
        this.xslMapping = new File(xslRootRealPath+"/xsltMapping.xml"); 
        
        tf = TransformerFactory.newInstance();
        DocumentBuilderFactory dfactory = DocumentBuilderFactory.newInstance();            
        dfactory.setNamespaceAware(true);
        
        try {
            db = dfactory.newDocumentBuilder();
            reload();
        } catch (Exception e) {
            throw new WebminException("Initialization Error for the XML Transformer\n"
                +e.getClass()+":"+e.getMessage());
        }
    }
    
    private void reload() throws SAXException, IOException {        
        doc = db.parse(xslMapping);        
        Element rootElement = doc.getDocumentElement();
        NodeList moduleNodeList = rootElement.getChildNodes();
        
        if (moduleNodeList != null) {
            moduleTable = new Hashtable(moduleNodeList.getLength());
            for (int i = 0; i < moduleNodeList.getLength(); i++) {
                Node moduleNode = moduleNodeList.item(i);
                if (!isTextNode(moduleNode)) {
                    NodeList fnNodeList = moduleNode.getChildNodes();
                    if (fnNodeList != null) {
                        Hashtable fnTable = new Hashtable(fnNodeList.getLength());
                        for (int j = 0; j < fnNodeList.getLength(); j++) {
                            Node fnNode = fnNodeList.item(j);
                            if (!isTextNode(fnNode)) {
                                String fnXSLPath = xslRootRealPath+"/"+fnNode.getFirstChild().getNodeValue();
                                fnTable.put(fnNode.getNodeName(), fnXSLPath);
                            }
                        }
                        moduleTable.put(moduleNode.getNodeName(), fnTable);
                    }
                }
            }
        }        
    }
    
    private boolean isTextNode(Node n) {
        if (n == null)
            return false;
        short nodeType = n.getNodeType();
        return nodeType == Node.CDATA_SECTION_NODE || nodeType == Node.TEXT_NODE;
    }
    
    String getTransformedHtml(String xml, String moduleName, String fnName)
        throws WebminException {
        String xsltPath = null;
        try {
            reload();
        } catch (Exception e) {
            throw new WebminException("Error format for the transformation file<br>"
                +e.getClass()+":"+e.getMessage());
        }
        Hashtable fnTable = null;
        if (moduleTable.get(moduleName) != null) {
            fnTable = (Hashtable)moduleTable.get(moduleName);
        }
        if (fnTable != null) {
            Object tmp = fnTable.get(fnName);
            if (tmp != null) {
                xsltPath = tmp.toString();
            }
        }
        
        StreamSource source = new StreamSource(new ByteArrayInputStream(xml.getBytes()));
        StreamSource template = new StreamSource(new File(xsltPath));
	StreamResult result = new StreamResult(new StringWriter());
        try {
            Transformer t = tf.newTransformer(template);        
            t.transform(source, result);
        } catch (Exception e) {
            throw new WebminException("Error when transforming XML file"+
                e.getClass()+":"+e.getMessage());
        }
        return result.getWriter().toString();       
    } 
}