package hk.hku.cecid.phoenix.hermes.webmin.servlet;

import java.io.Writer;
import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.Enumeration;

import hk.hku.cecid.phoenix.common.util.Property;
import hk.hku.cecid.phoenix.message.handler.DiagTool;
import javax.servlet.http.HttpServletRequest;

import org.apache.xpath.XPathAPI;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

// Imported JAVA API for XML Parsing 1.0 classes
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;

// Imported Serializer classes
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.dom.DOMSource;


class DiagnosisDumpHandler {
    private String DIAG_DUMP_CONFIG_FILE = "diagnosis.properties.xml";
    private String SERVLET_CONFIG_FILE = "webminServlet.properties.xml";
        
    private File configFile;
    private Document doc;
    private Element rootElement;
    private Node criteriaNode;
    private Node appContextsNode;
    private DocumentBuilderFactory dfactory;
    private DocumentBuilder db;
    private TransformerFactory tFactory;
    private Transformer transformer;
    private String dumpRootUrl;
    private String cpaIdPrefix = "CPAId";
    private String convIdPrefix = "ConversationId";
    private String servicePrefix = "Service";
    private String actionPrefix = "Action";
    private Property property;
    
    DiagnosisDumpHandler() throws Exception {            
        Property prop = Property.load(SERVLET_CONFIG_FILE);
        this.dumpRootUrl = prop.get("DumpRootUrl");
        configFile = ConfigFileFinder.findFile(DIAG_DUMP_CONFIG_FILE);
            
        dfactory = DocumentBuilderFactory.newInstance();
        dfactory.setNamespaceAware(true);            
        db = dfactory.newDocumentBuilder();                        
        tFactory = TransformerFactory.newInstance();
        transformer = tFactory.newTransformer();        
        property = Property.load(DIAG_DUMP_CONFIG_FILE);
    }
    
    private void init() throws Exception{
        doc = db.parse(configFile);
        rootElement = doc.getDocumentElement();
        
        criteriaNode = XPathAPI.selectSingleNode(doc, "//Criteria");
        Node timeNode = XPathAPI.selectSingleNode(criteriaNode, "//TimePeriod");
        appContextsNode = XPathAPI.selectSingleNode(criteriaNode, "//AppContexts");
        
        if (timeNode != null) {
            criteriaNode.removeChild(timeNode);
        }
        if (appContextsNode != null) {
            criteriaNode.removeChild(appContextsNode);                    
        } 
        Element appContextsElement = doc.createElement("AppContexts");	
        appContextsNode = criteriaNode.appendChild(appContextsElement);  
    }
    
    private void setTimeCriteria(String startTime, String endTime) {
        Element timePeriodElement = doc.createElement("TimePeriod"); 	
        if (startTime != null) {            
            Element startTimeElement = doc.createElement("StartTime");
            startTimeElement.appendChild(doc.createTextNode(startTime));
            timePeriodElement.appendChild(startTimeElement);
        }
        
        if (endTime != null) {
            Element endTimeElement = doc.createElement("EndTime");
            endTimeElement.appendChild(doc.createTextNode(endTime));
            timePeriodElement.appendChild(endTimeElement);        
        }
        criteriaNode.appendChild(timePeriodElement);     
    }    
    
    private void addAppContext(String cpaID, String convID, String service, String action) {
        Element appContextElement = doc.createElement("AppContext"); 	    
        Element cpaIdElement = doc.createElement("CpaId"); 	
        Element conversationIdElement = doc.createElement("ConversationId");
        Element serviceElement = doc.createElement("Service"); 	
        Element actionElement = doc.createElement("Action");
        cpaIdElement.appendChild(doc.createTextNode(cpaID));
        conversationIdElement.appendChild(doc.createTextNode(convID));
        serviceElement.appendChild(doc.createTextNode(service));
        actionElement.appendChild(doc.createTextNode(action));        
        appContextElement.appendChild(cpaIdElement);
        appContextElement.appendChild(conversationIdElement);
        appContextElement.appendChild(serviceElement);
        appContextElement.appendChild(actionElement);
        appContextsNode.appendChild(appContextElement);        
    }
    
    String dump(HttpServletRequest request) throws Exception{
        init();
        int i = 0;
        while(request.getHeader(cpaIdPrefix+i) != null &&
            request.getHeader(convIdPrefix+i) != null &&
            request.getHeader(servicePrefix+i) != null &&
            request.getHeader(actionPrefix+i) != null) {
            addAppContext(request.getHeader(cpaIdPrefix+i),
                request.getHeader(convIdPrefix+i),
                request.getHeader(servicePrefix+i),
                request.getHeader(actionPrefix+i));
            i++;
        }
        String startDate = null;
        String endDate = null;
        boolean dumpDB = request.getHeader("DumpDatabase") != null &&
                            request.getHeader("DumpDatabase").equals("True");
        boolean dumpLog = request.getHeader("DumpLog") != null &&
                            request.getHeader("DumpLog").equals("True");
        boolean dumpFile = request.getHeader("DumpFiles") != null &&
                            request.getHeader("DumpFiles").equals("True");
        if (request.getHeader("StartDate") != null) {
            startDate = request.getHeader("StartDate");
        }
        if (request.getHeader("EndDate") != null) {
            endDate = request.getHeader("EndDate");
        }
        setTimeCriteria(startDate, endDate);
        saveConfigFile();
        property.reload();
        DiagTool tool = new DiagTool(property);
        if (dumpDB) {
            tool.dumpDatabase();
        }
        if (dumpLog) {
            tool.dumpLog();
        }
        if (dumpFile) {
            tool.dumpFiles();
        }
        String resultLinks = getDumpLinks(property, dumpDB, 
            dumpLog, dumpFile);        
        return resultLinks;
    }
    
        
    private String getDumpLinks(Property property, boolean dumpDB,
        boolean dumpLog, boolean dumpFile) {        
        
        StringBuffer buffer = new StringBuffer();
        String dumpRoot = property.get("Diagnosis/Destination");
        buffer.append("<DiagonsisDump>");
        if (dumpDB) {            
            File dumpDBRoot = new File(dumpRoot+"/database");
            if (dumpDBRoot.isDirectory()) {
                buffer.append("<DataBaseDumpFiles>");
                File[] dbFiles = dumpDBRoot.listFiles();
                for (int i = 0; i < dbFiles.length; i++) {                    
                    buffer.append("<File>");
                    buffer.append(dumpRootUrl);
                    buffer.append("/");
                    buffer.append("database");
                    buffer.append("/");
                    buffer.append(dbFiles[i].getName());
                    buffer.append("</File>");
                }
                buffer.append("</DataBaseDumpFiles>");
            }
        }
        
        if (dumpLog) {
            File dumpLogRoot = new File(dumpRoot+"/logs");
            if (dumpLogRoot.isDirectory()) {
                File[] logFiles = dumpLogRoot.listFiles();
                buffer.append("<MshLogFiles>");
                for (int i = 0; i < logFiles.length; i++) {                    
                    buffer.append("<File>");
                    buffer.append(dumpRootUrl);
                    buffer.append("/");
                    buffer.append("logs");
                    buffer.append("/");
                    buffer.append(logFiles[i].getName());
                    buffer.append("</File>");                    
                }
                buffer.append("</MshLogFiles>");               
            }
        }
        
        if (dumpFile) {
            String[] props = property.getMultiple("Diagnosis/Copy");        
            if (props != null) {
                buffer.append("<SystemFiles>");
                int length = props.length;                            
                for (int i = 0; i < length; i++) {                    
                    String prefix = "Diagnosis/Copy[" + i + "]";
                    String todir = property.get(prefix + "/ToDir");
                    
                    buffer.append("<File>");
                    buffer.append(dumpRootUrl+"/"+todir);
                    buffer.append("</File>");                    
                }
                buffer.append("</SystemFiles>");
            }            
        }
        buffer.append("</DiagonsisDump>");            
        return buffer.toString();
    }
        
    private void saveConfigFile() throws Exception {        
        DOMSource source = new DOMSource(doc);
        StreamResult result = new StreamResult(configFile);
        transformer.transform(source, result);        
    }
    
    
}