package hk.hku.cecid.phoenix.hermes.webmin.servlet;

import java.io.IOException;
import java.io.StringWriter;

import java.net.URL;
import java.net.MalformedURLException;
import java.util.Date;
import java.util.HashSet;
import java.util.Map;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.Locale;
import java.text.DateFormat;
import java.text.ParseException;

import javax.servlet.http.HttpServletRequest;

import hk.hku.cecid.phoenix.common.util.Property;
import hk.hku.cecid.phoenix.message.handler.MessageServiceHandler;
import hk.hku.cecid.phoenix.message.handler.Request;
import hk.hku.cecid.phoenix.message.handler.ApplicationContext;
import hk.hku.cecid.phoenix.message.handler.RequestException;
import hk.hku.cecid.phoenix.message.handler.Utility;

import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

class MshAdmin {    
    
    private Property property;
    private Request adminRequest;
    
    private DateFormat dateFormat = DateFormat.getDateTimeInstance
            (DateFormat.MEDIUM, DateFormat.MEDIUM, Locale.US);
    private DiagnosisDumpHandler dumpHandler;
    
    private final String cpaIdPrefix = "CPAId";
    private final String convIdPrefix = "ConversationId";
    private final String servicePrefix = "Service";
    private final String actionPrefix = "Action";
    private final String messageIdPrefix = "MessageId";    
    private DocumentBuilder documentBuilder;
    private Transformer transformer;
    
    MshAdmin() throws Exception {
        dumpHandler = new DiagnosisDumpHandler();        
        DocumentBuilderFactory factory =DocumentBuilderFactory.newInstance();
        documentBuilder = factory.newDocumentBuilder();
        TransformerFactory tfactory =TransformerFactory.newInstance();
        transformer = tfactory.newTransformer();
    }
    
        
    String archiveByAppContext(HttpServletRequest request) throws Exception{        
        HashSet set = new HashSet();
        int i = 0;
        while(request.getHeader(cpaIdPrefix+i) != null &&
            request.getHeader(convIdPrefix+i) != null &&
            request.getHeader(servicePrefix+i) != null &&
            request.getHeader(actionPrefix+i) != null) {
            set.add(new ApplicationContext(request.getHeader(cpaIdPrefix+i), 
                request.getHeader(convIdPrefix+i),
                request.getHeader(servicePrefix+i),
                request.getHeader(actionPrefix+i)));
            i++;
        }
                
        Object[] tmp = set.toArray();
        ApplicationContext[] appContexts = new ApplicationContext[tmp.length];
        for (int j = 0; j < tmp.length; j++) {
            appContexts[j] = (ApplicationContext)tmp[j];
        }
        return (getRequest().archiveByAppContext(appContexts));                        
    }
            
    String archiveByDate(HttpServletRequest request) throws Exception {        
        String tmp = null;
        Date startTime = null;
        Date endTime = null;
        
        startTime = Utility.fromUTCString(request.getHeader("StartDate"));
        endTime = Utility.fromUTCString(request.getHeader("EndDate"));
        return (getRequest().archiveByDate(startTime, endTime));        
    }
            
    String backupMsh() throws Exception {       
        return (getRequest().backupMSH());        
    }
    
            
    String checkDatabase() throws Exception {        
        return (getRequest().checkDatabase());        
    }
    
    String checkInternalStates() throws Exception {        
        return (getRequest().checkInternalStates());                    
    }
    
    String checkPersistence() throws Exception {         
        return (getRequest().checkPersistence());        
    }
    
    String deletePendingMessages(HttpServletRequest request) throws Exception {
        ArrayList idList = new ArrayList();
        int i = 0;
        while(request.getHeader(messageIdPrefix+i) != null) {
            idList.add(request.getHeader(messageIdPrefix+i));
            i++;
        }
        Map tmp = getRequest().deletePendingMessages(idList);
        Iterator itr = tmp.keySet().iterator();
        Document document = documentBuilder.newDocument();
        Element elem_root = document.createElement("Messages");
        while(itr.hasNext()) {
            String msgId = itr.next().toString();
            String state = tmp.get(msgId).toString();
            Element elem_message = document.createElement("Message");
            elem_message.setAttribute("id", msgId);
            Element elem_state = document.createElement("State");
            elem_state.appendChild(document.createTextNode(state));
            elem_message.appendChild(elem_state);
            elem_root.appendChild(elem_message);
        }
        document.appendChild(elem_root);
        return toXMLString(document);                
    }
    
    String diagonsisDump(HttpServletRequest request) throws Exception {
        return (dumpHandler.dump(request));
    }
    
    String getDBConnectionPoolInfo() throws Exception {        
        return (getRequest().getDBConnectionPoolInfo());        
    }
    
    String getIsHalted() throws Exception {        
        Document document = documentBuilder.newDocument();
        Element elem_root = document.createElement("Result");
        Element elem_isHalted = document.createElement("IsHalted");
        boolean result = getRequest().getIsHalted();
        if (result) {                    
            elem_isHalted.appendChild(document.createTextNode("Yes"));
        } else {
            elem_isHalted.appendChild(document.createTextNode("No"));
        }
        elem_root.appendChild(elem_isHalted);
        document.appendChild(elem_root);        
        return toXMLString(document);
    }
    
    String getMessageStatus(HttpServletRequest request) throws Exception {
        ArrayList idList = new ArrayList();
        int i = 0;
        while(request.getHeader(messageIdPrefix+i) != null) {
            idList.add(request.getHeader(messageIdPrefix+i));
            i++;
        }
        return (getRequest().getMessageStatus(idList));
    }
    
    String getNumRecordsInDB() throws Exception {        
        return (getRequest().getNumRecordsInDB());        
    }
    
    
    String getPendingMessages() throws Exception {                
        String[] tmp = getRequest().getPendingMessages();
        Document document = documentBuilder.newDocument();
        Element elem_root = document.createElement("Messages");
        if (tmp != null && tmp.length > 0) {            
            for(int i = 0; i < tmp.length; i++) {        
                Element elem_message = document.createElement("Message");
                elem_message.setAttribute("id", tmp[i]);                
                elem_root.appendChild(elem_message);
            }
        }
        document.appendChild(elem_root);       
        return toXMLString(document);
    }
    
    
    String getTrustedRepository() throws Exception {
        String[] tmp = getRequest().getTrustedRepository();
        Document document = documentBuilder.newDocument();
        Element elem_root = document.createElement("TrustedRepositories");
        if (tmp != null && tmp.length > 0) {            
            for(int i = 0; i < tmp.length; i++) {        
                Element elem_repo = document.createElement("TrustedRepository");
                elem_repo.appendChild(document.createTextNode(tmp[i]));                
                elem_root.appendChild(elem_repo);
            }
        }
        document.appendChild(elem_root);       
        return toXMLString(document);        
    }
    
    String suspendMsh() throws Exception {
        return (getRequest().haltMSH(Request.HALT_SUSPEND));        
    }
    
    String terminateMsh() throws Exception {
        return (getRequest().haltMSH(Request.HALT_TERMINATE));        
    }
    
    String reportEnvironment() throws Exception {        
        return (getRequest().reportEnvironment());        
    }
    
    String resetConnectionPool() throws Exception {
        return (getRequest().resetConnectionPool());            
    }
    
    String restoreMsh() throws Exception {
        return (getRequest().restoreMSH());        
    }
    
    String resumeMsh() throws Exception {        
        return (getRequest().resumeMSH());            
    }
            
    String testLoopback() throws Exception {
        return (getRequest().testLoopback());        
    }
    
    String unregister(HttpServletRequest request) throws Exception {
        Request mshRequest = getRequest(
            new ApplicationContext(request.getHeader(cpaIdPrefix+"0"),
                 request.getHeader(convIdPrefix+"0"),
                 request.getHeader(servicePrefix+"0"),
                 request.getHeader(actionPrefix+"0")));
        Document document = documentBuilder.newDocument();
        Element elem_root = document.createElement("Result");
        Element elem_unreg = document.createElement("Unregister");
        boolean result = mshRequest.unregister();
        if (result) {
            elem_unreg.appendChild(document.createTextNode("Success"));
        } else {
            elem_unreg.appendChild(document.createTextNode("Failure"));
        }
        elem_root.appendChild(elem_unreg);
        document.appendChild(elem_root);       
        return toXMLString(document);        
    }
    
    String getMshVersion() throws Exception{
        Document document = documentBuilder.newDocument();
        Element elem_root = document.createElement("Result");
        Element elem_version = document.createElement("MshVersion");
        elem_version.appendChild(document.createTextNode(
            MessageServiceHandler.getMetaData().getRelease()));        
        elem_root.appendChild(elem_version);
        document.appendChild(elem_root);               
        return toXMLString(document);
    }
              
    private Request getRequest(ApplicationContext appCxt) {
        Request mshRequest = null;
        URL dummyToMshUrl = null;
        try {
            dummyToMshUrl = new URL("http://www.sysAdministrtaor.com");
        } catch (MalformedURLException e) {          
            e.printStackTrace();
            System.exit(-1);
        }
        String type = "HTTP";
                
               
        try {
            mshRequest = new Request(appCxt, dummyToMshUrl, null, type);            
        } catch (RequestException e) {
            e.printStackTrace();
            return null;
        }
        return mshRequest;
    }
    
    private Request getRequest() {
        if (adminRequest == null) {
            URL dummyToMshUrl = null;
            try {
                dummyToMshUrl = new URL("http://www.sysAdministrtaor.com");
            } catch (MalformedURLException e) {          
                e.printStackTrace();
                System.exit(-1);
            }
            
            String type = "HTTP";
            try {        
                adminRequest = new Request(dummyToMshUrl, null, type);            
            } catch (RequestException e) {
                e.printStackTrace();
                return null;
            }
            return adminRequest;
        } else {
            return adminRequest;
        }
    }    
    
    private String toXMLString(Document doc) throws Exception {
        DOMSource source = new DOMSource(doc);
        StringWriter writer = new StringWriter();
        StreamResult result = new StreamResult(writer);
        transformer.transform(source, result);
        return writer.toString();
    }        
}