package hk.hku.cecid.phoenix.hermes.webmin.servlet;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import hk.hku.cecid.phoenix.common.util.Property;

import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;


public class RequestHandler extends HttpServlet {
    private final String MSH_CLIENT_PROPERTIES = "msh_client.properties.xml";
    
    public static String ENV_CHECK = "ENV_CHECK";      
    public static String MSG_TRACK = "MSG_TRACK";      
    public static String DIAGNOSIS = "DIAGNOSIS";                
    public static String MAINTENANCE = "MAINTENANCE";      
    public static String ARCHIVE = "ARCHIVE";
    public static String DB_OPERATION = "DB_OPERATION";
    public static String AUTHENTICATE = "AUTHENTICATE";
        
    private MshAdmin mshAdmin;
    private PrintWriter pw;
    private HttpServletResponse response;
    private HttpServletRequest request;
    private DocumentBuilder documentBuilder;
    private Transformer transformer;
        
    private void initUtil() throws Exception {
        if (documentBuilder == null) {
            DocumentBuilderFactory factory =DocumentBuilderFactory.newInstance();
            documentBuilder = factory.newDocumentBuilder();
        } 
        if (transformer == null) {
            TransformerFactory tfactory =TransformerFactory.newInstance();
            transformer = tfactory.newTransformer();
        }
    }
        
    private void initMshAdmin() throws Exception {
        if (mshAdmin == null) {
            mshAdmin = new MshAdmin();
        }
    }
          
    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException {
        doPost(request, response);
    }

    public void doPost(HttpServletRequest request,
                       HttpServletResponse response)
        throws ServletException, IOException {
        //setup global request and response
        pw = response.getWriter();
        this.response = response;
        this.request = request;
        
        //authentication
        authenticate();
        
        //initialize mshAdmin, if it has not been initialized
        try {
            initMshAdmin();
        } catch (Exception e) {
            sendError("Error in initializing MshAdmin for webmin servlet<br>"+
                e.getClass()+":"+e.getMessage());
        }
        
        //initialize mshAdmin, if it has not been initialized
        try {
            initUtil();
        } catch (Exception e) {
            sendError("Error in initializing transformer and document builder for webmin servlet<br>"+
                e.getClass()+":"+e.getMessage());
        }    
        
        //check for requesting module and function name
        String moduleName = request.getHeader("ModuleName");
        String functionName = request.getHeader("FunctionName");
        
        //perform operation base on request module and function
        if (functionName != null && moduleName != null) {
            if (moduleName.equals(ENV_CHECK)) {
                process_ENV_CHECK(functionName);
            } else if (moduleName.equals(MSG_TRACK)) {
                process_MSG_TRACK(functionName);
            } else if (moduleName.equals(DIAGNOSIS)) {
                process_DIAGNOSIS(functionName);
            } else if (moduleName.equals(MAINTENANCE)) {
                process_MAINTENANCE(functionName);
            } else if (moduleName.equals(ARCHIVE)) {
                process_ARCHIVE(functionName);
            } else if (moduleName.equals(DB_OPERATION)) {
                process_DB_OPERATION(functionName);
            } else if (moduleName.equals(AUTHENTICATE)) {
                process_AUTHENTICATE();
            }
        } else {
            sendError("Malformed request for the webmin servlet<br>");
        }
    }
    
    private void process_AUTHENTICATE() {
        Document document = documentBuilder.newDocument();
        Element elem_root = document.createElement("Result");
        elem_root.appendChild(document.createTextNode("Authenticated"));        
        document.appendChild(elem_root);
        try {
            sendResult(toXMLString(document));
        } catch (Exception e) {
            sendError("Exception occurs when transforming authentication XML result<br>"+
                e.getClass()+":"+e.getMessage());
        }
    }
    
    private void process_ENV_CHECK(String fnName){
        try {
            if (fnName.equals("getIsHalted")) {
                sendResult(mshAdmin.getIsHalted());
            } else if (fnName.equals("reportEnvironment")) {
                sendResult(mshAdmin.reportEnvironment());
            } else if (fnName.equals("getTrustedRepository")) {
                sendResult(mshAdmin.getTrustedRepository());
            } else if (fnName.equals("getMshVersion")) {
                sendResult(mshAdmin.getMshVersion());
            } else {
                sendError("Requesting operation has not been implemented for the webmin servlet<br>");
            }
        } catch (Exception e) {
            sendError("Exception occurs when performing operation '"+fnName+"' in the webmin servlet<br>"+
                e.getClass()+":"+e.getMessage());
        }
    }
    
    private void process_DB_OPERATION(String fnName){
        try {
            if (fnName.equals("checkDatabase")) {
                sendResult(mshAdmin.checkDatabase());
            } else if (fnName.equals("getDBConnectionPoolInfo")) {
                sendResult(mshAdmin.getDBConnectionPoolInfo());
            } else if (fnName.equals("resetConnectionPool")) {
                sendResult(mshAdmin.resetConnectionPool());
            } else if (fnName.equals("getNumRecordsInDB")) {
                sendResult(mshAdmin.getNumRecordsInDB());
            } else {
                sendError("Requesting operation has not been implemented for the webmin servlet<br>");
            }
        } catch (Exception e) {
            sendError("Exception occurs when performing operation '"+fnName+"' in the webmin servlet<br>"+
                e.getClass()+":"+e.getMessage());
        }
    }
    
    private void process_MSG_TRACK(String fnName){
        try {
            if (fnName.equals("getPendingMessages")) {
                sendResult(mshAdmin.getPendingMessages());
            } else if (fnName.equals("getMessageStatus")) {
                sendResult(mshAdmin.getMessageStatus(request));
            } else if (fnName.equals("deletePendingMessages")) {
                sendResult(mshAdmin.deletePendingMessages(request));
            } else {
                sendError("Requesting operation has not been implemented for the webmin servlet<br>");
            }
        } catch (Exception e) {
            sendError("Exception occurs when performing operation '"+fnName+"' in the webmin servlet<br>"+
                e.getClass()+":"+e.getMessage());
        }
    }
    
    private void process_DIAGNOSIS(String fnName){
        try {
            if (fnName.equals("diagnosisDump")) {
                sendResult(mshAdmin.diagonsisDump(request));
            } else {
                sendError("Requesting operation has not been implemented for the webmin servlet<br>");
            }
        } catch (Exception e) {
            sendError("Exception occurs when performing operation '"+fnName+"' in the webmin servlet<br>"+
                e.getClass()+":"+e.getMessage());
        }
    }
    
    private void process_MAINTENANCE(String fnName){
        try {
            if (fnName.equals("suspendMsh")) {
                sendResult(mshAdmin.suspendMsh());
            } else if (fnName.equals("terminateMsh")) {
                sendResult(mshAdmin.terminateMsh());
            } else if (fnName.equals("resumeMsh")) {
                sendResult(mshAdmin.resumeMsh());
            } else {
                sendError("Requesting operation has not been implemented for the webmin servlet<br>");
            }
        } catch (Exception e) {
            sendError("Exception occurs when performing operation '"+fnName+"' in the webmin servlet<br>"+
                e.getClass()+":"+e.getMessage());
        }    
    }
    
    private void process_ARCHIVE(String fnName){
        try {
            if (fnName.equals("archiveByDate")) {
                sendResult(mshAdmin.archiveByDate(request));
            } else if (fnName.equals("archiveByAppContext")) {
                sendResult(mshAdmin.archiveByAppContext(request));
            } else {
                sendError("Requesting operation has not been implemented for the webmin servlet<br>");
            }
        } catch (Exception e) {
            sendError("Exception occurs when performing operation '"+fnName+"' in the webmin servlet<br>"+
                e.getClass()+":"+e.getMessage());
        }     
    }
    
    private void authenticate() {
        Property prop = null;
        try {
            prop = Property.load(MSH_CLIENT_PROPERTIES);
        } catch (IOException e) {
            sendError("Config file msh_client.properties.xml cannot be found<br>"+
                e.getClass()+":"+e.getMessage());
        }
        String authName = request.getHeader("authenticate_user");
        String authPwd = request.getHeader("authenticate_password");
        if (authName == null || authPwd == null || 
            !authName.equals(prop.get("Request/Config/UserName")) ||
            !authPwd.equals(prop.get("Request/Config/Password"))) {
            sendError("Fail to authenticate with the webmin servlet."+
                " Please contact your system administrator to change the user name and password in the webminApp.properties.xml for authentication<br>");
        }
    }
    
    private void sendResult(String resultMsg) {
        response.setHeader("ResponseResult", "Normal");
        pw.print(resultMsg);
        pw.flush();
        pw.close();
        return;
    }
    
    
    private void sendError(String errorMsg) {
        response.setHeader("ResponseResult", "Error");
        pw.print(errorMsg);
        pw.flush();
        pw.close();
        return;
    }
    
    private String toXMLString(Document doc) throws Exception {        
        DOMSource source = new DOMSource(doc);
        StringWriter writer = new StringWriter();
        StreamResult result = new StreamResult(writer);
        transformer.transform(source, result);
        return writer.toString();
    }
        
}